<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Followupemail2
 * @version    2.2.3
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Followupemail2\Model\Sample\Converter;

/**
 * Class Xml
 * @package Aheadworks\Followupemail2\Model\Sample\Converter
 * @codeCoverageIgnore
 */
class Xml implements \Magento\Framework\Config\ConverterInterface
{
    /**
     * {@inheritdoc}
     */
    public function convert($source)
    {
        $output = [];
        if (!$source instanceof \DOMDocument) {
            return $output;
        }

        $tagNames = [
            'campaign' => [
                'event' => [
                    'email' => []
                ]
            ]
        ];

        $output = $this->convertElementsByTagNames($source, $tagNames);

        return $output;
    }

    /**
     * Convert elements by tag name
     *
     * @param \DOMDocument $source
     * @param array $tags
     * @param string|null $tag
     * @return array
     */
    private function convertElementsByTagNames($source, $tags, $tag = null)
    {
        $output = [];
        foreach ($tags as $tagName => $tagChildren) {
            if ($tag && $tag != $tagName) {
                continue;
            }
            $elements = $source->getElementsByTagName($tagName);
            foreach ($elements as $element) {
                $elementData = [];
                /** @var $element \DOMElement */
                foreach ($element->childNodes as $child) {
                    if (!$child instanceof \DOMElement) {
                        continue;
                    }
                    $elementData[$child->nodeName] = $child->nodeValue;
                }

                if (is_array($tagChildren)) {
                    foreach ($tagChildren as $tagName => $tagContent) {
                        $elementData[$tagName] = $this->convertElementsByTagNames($element, $tagChildren, $tagName);
                    }
                    $output[] = $elementData;
                } else {
                    $output = $elementData;
                }
            }
        }
        return $output;
    }
}
